@ECHO OFF
SETLOCAL
TITLE Introscope -- Convert alert actions into Transaction Tracer
REM This script is called by the Enterprise Manager to convert danger
REM alerts to a Transaction Tracer session with appropriate threshold

REM ****
REM Sets the current working directory with quotes around it
REM This enables this file to execute correctly if it is
REM invoked somewhere other than its location.
REM ****
FOR /f "tokens=1 delims=" %%a in ('cd') DO SET PWD=%%a

REM ****
REM Set the command directory to the command argument itself,
REM namely the full path of this file or the relative path
REM to where the file was invoked.
SET CMD_DIR=%0
REM Now simply strip out the name of this file to get dir
REM !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
REM        Use current filename
SET CMD_DIR=%CMD_DIR:alert2TxnTracer.cmd=%
REM !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
REM Strip the quotes
REM add quotes if they're not there
SET CMD_DIR="%CMD_DIR%"
REM if not add quotes, then this statement chokes
SET CMD_DIR=%CMD_DIR:"=%
REM if the current dir wasn't determined, just set it to working dir
IF NOT EXIST "%CMD_DIR%" SET CMD_DIR=.
IF "%CMD_DIR%" == %0 SET CMD_DIR=.
IF "%CMD_DIR%" == "%0" SET CMD_DIR=.

REM ****
REM Now set the home directory for Introscope, and others
IF NOT EXIST "%CMD_DIR%\setIscEnv.cmd" goto noSetEnvFile
call "%CMD_DIR%\setIscEnv.cmd"

SET LOG_FILE=%ISC_LOGS%\alert2TxnTracer.log

REM Sample alert:
REM SET ALERT="6/7/04 5:24:15 PM PDT Introscope Enterprise Manager (lemma.wilytech.com:172.27.0.104) reported: The Alert Test exceeded danger target of 91: SuperDomain|Custom Metric Host (Virtual)|Custom Metric Process (Virtual)|Custom Metric Agent (Virtual)|Test:Count = 0"
REM Actions get passed parameters or alert in one long string, with double quotes.
REM Tried stripping quotes with DOS utility, but special chars in alert threw it off.
SET ALERT=%1
REM use single quotes because alert already has double quotes from arg
if 'X%ALERT%' == 'X' goto noAlert

REM Get the danger target value
SET TEMP=%ALERT:danger target of =~%
REM use single quotes because alert already has double quotes from arg
if '%TEMP%' == '%ALERT%' goto noDanger
REM don't wrap temp in quotes, already has them
FOR /f "tokens=2 delims=~" %%a in (%TEMP%) DO SET TEMP="%%a"
FOR /f "delims=: " %%a in (%TEMP%) DO SET TT_THRESHOLD_IN_MSECS=%%a
REM Get the agent, the second to last record separate by pipes
FOR /f "tokens=1-5 delims=|" %%a in (%TEMP%) DO SET TT_AGENTS=%%d
IF "X%TT_THRESHOLD_IN_MSECS%" == "X" GOTO noThreshold 
IF /i %TT_THRESHOLD_IN_MSECS% LEQ 0 GOTO badThreshold

REM Run for 15 minutes
SET TT_DURATION_IN_SECS=900
REM testing -- SET TT_DURATION_IN_SECS=1

REM Java Options (e.g. increase Max heap size )
SET JAVA_OPTS=-Xms64m -Xmx512m
SET JAVA_OPTS=%JAVA_OPTS% %CLW_USER_OPT% %CLW_PASSWORD_OPT% %CLW_EM_HOST_OPT% %CLW_EM_PORT_OPT%

SET COMMAND=trace transactions exceeding %TT_THRESHOLD_IN_MSECS% milliseconds in agents matching \"%TT_AGENTS%\" for %TT_DURATION_IN_SECS% seconds

SET AGENT_STR=%TT_AGENTS%
REM Remove the spaces, parentheses
SET AGENT_STR=%AGENT_STR:(=%
SET AGENT_STR=%AGENT_STR:)=%
SET AGENT_STR=%AGENT_STR: =%
if "X%TT_AGENTS%" == "X(.*)" SET AGENT_STR=AllAgents
FOR /f "tokens=1-4 delims=:." %%a in ("%time%") DO SET SECS_STR=%%a%%b%%c%%d
FOR /f "tokens=1-4 delims=/ " %%a in ("%date%") DO SET TRACES_DIR=%ISC_TRACES%\%%d\%%b\%%c

REM Create the traces dir, if not exits
IF NOT EXIST "%TRACES_DIR%" MKDIR "%TRACES_DIR%"
SET TT_FILE=%TRACES_DIR%\%AGENT_STR%_%TT_THRESHOLD_IN_MSECS%msecs_for%TT_DURATION_IN_SECS%secs.%SECS_STR%.xml

REM send stderr to null b/c there are meaningless disconnect excepts that might be *scary-looking*
ECHO %date% %time%: Saving sessoutput to %TT_FILE% >> "%LOG_FILE%"
"%ISC_JAVA_HOME%\bin\java" %JAVA_OPTS% -jar "%CLW_JAR%" %COMMAND% > "%TT_FILE%" 2>NUL
goto finish

:noAlert
ECHO %date% %time%: No alert messaged passed. >> "%LOG_FILE%"
goto finish

:noThreshold 
ECHO %date% %time%: Could not determine danger target >> "%LOG_FILE%"
ECHO %date% %time%: Alert: %ALERT% >> "%LOG_FILE%"
goto finish

:badThreshold 
ECHO %date% %time%: Danger target less than zero: %TT_THRESHOLD_IN_MSECS% >> "%LOG_FILE%"
ECHO Alert: %ALERT% >> "%LOG_FILE%"
goto finish

:noDanger
ECHO %date% %time%: No match on alert for reaching danger: %ALERT% >> "%LOG_FILE%"
goto finish

:noSetEnvFile
ECHO There is no environment setup file: setIscEnv.cmd
ECHO Please re-run configureWily.cmd in installation directory!
goto finish

:finish
ENDLOCAL
